/*
  One MAX7219 connected to an 8x8 LED matrix.
 */

#include <avr/io.h>
#include <util/delay.h>
#include "lcd.h"

#define CLK_HIGH1()  PORTB |= (1<<PB1)
#define CLK_LOW1()   PORTB &= ~(1<<PB1)
#define CS_HIGH1()   PORTB |= (1<<PB2)
#define CS_LOW1()    PORTB &= ~(1<<PB2)
#define DATA_HIGH1() PORTB |= (1<<PB3)
#define DATA_LOW1()  PORTB &= ~(1<<PB3)
#define INIT_PORT1() DDRB |= (1<<PB3) | (1<<PB1) | (1<<PB2)


#define CLK_HIGH2()  PORTB |= (1<<PB5)
#define CLK_LOW2()   PORTB &= ~(1<<PB5)
#define CS_HIGH2()   PORTB |= (1<<PB6)
#define CS_LOW2()    PORTB &= ~(1<<PB6)
#define DATA_HIGH2() PORTB |= (1<<PB7)
#define DATA_LOW2()  PORTB &= ~(1<<PB7)
#define INIT_PORT2() DDRB |= (1<<PB5) | (1<<PB6) | (1<<PB7)



uint8_t left_cursor_x = 0;
uint8_t left_cursor_y = 0;
uint8_t show_left_cursor = 1;

	
	
uint8_t right_cursor_x = 0;
uint8_t right_cursor_y = 0;
uint8_t show_right_cursor = 1;

uint8_t right_plane[8][8];
uint8_t left_plane[8][8];

uint8_t hit_matrix_right[8][8];
uint8_t hit_matrix_left[8][8];

uint8_t orientation_right = 0;
uint8_t orientation_left = 0;



const __flash uint8_t smile[8] = {
	0b00000000,
	0b01100110,
	0b01100110,
	0b00011000,
	0b00011000,
	0b10000001,
	0b01000010,
0b00111100};

const __flash uint8_t sad[8] = {
	0b00000000,
	0b01100110,
	0b01100110,
	0b00011000,
	0b00011000,
	0b00000000,
	0b00111100,
	0b01000010,
};

void initialise_planes() {
	
	uint8_t i, j;
	
	for (i = 0; i < 8; i++) {
		for (j = 0; j < 8; j++) {
			right_plane[i][j] = 0;
			left_plane[i][j] = 0;
			hit_matrix_left[i][j] = 0;
			hit_matrix_right[i][j] = 0;
			
		}		
	}
}


void spi_send1(uint8_t data)
{
    uint8_t i;

    for (i = 0; i < 8; i++, data <<= 1)
    {
	CLK_LOW1();
	if (data & 0x80)
	    DATA_HIGH1(); 
	else
	    DATA_LOW1();
	CLK_HIGH1();
    }
    
}

void max7219_writec1(uint8_t high_byte, uint8_t low_byte)
{
    CS_LOW1();
    spi_send1(high_byte);
    spi_send1(low_byte);
    CS_HIGH1();
}

void spi_send2(uint8_t data)
{
	uint8_t i;

	for (i = 0; i < 8; i++, data <<= 1)
	{
		CLK_LOW2();
		if (data & 0x80)
		DATA_HIGH2();
		else
		DATA_LOW2();
		CLK_HIGH2();
	}
	
}

void max7219_writec2(uint8_t high_byte, uint8_t low_byte)
{
	CS_LOW2();
	spi_send2(high_byte);
	spi_send2(low_byte);
	CS_HIGH2();
}


void max7219_clear1(void)
{
    uint8_t i;
    for (i = 0; i < 8; i++)
    {
	max7219_writec1(i+1, 0);
    }
}

void max7219_clear2(void)
{
	uint8_t i;
	for (i = 0; i < 8; i++)
	{
		max7219_writec2(i+1, 0);
	}
}



void max7219_init1(void)
{
    INIT_PORT1();
    // Decode mode: none
    max7219_writec1(0x09, 0);
    // Intensity: 3 (0-15)
    max7219_writec1(0x0A, 1);
    // Scan limit: All "digits" (rows) on
    max7219_writec1(0x0B, 7);
    // Shutdown register: Display on
    max7219_writec1(0x0C, 1);
    // Display test: off
    max7219_writec1(0x0F, 0);
    max7219_clear1();
}


void max7219_init2(void)
{
	INIT_PORT2();
	// Decode mode: none
	max7219_writec2(0x09, 0);
	// Intensity: 3 (0-15)
	max7219_writec2(0x0A, 1);
	// Scan limit: All "digits" (rows) on
	max7219_writec2(0x0B, 7);
	// Shutdown register: Display on
	max7219_writec2(0x0C, 1);
	// Display test: off
	max7219_writec2(0x0F, 0);
	max7219_clear2();
}



uint8_t display1[8];
uint8_t display2[8];

void update_display1(void)
{
    uint8_t i;

    for (i = 0; i < 8; i++)
    {
	max7219_writec1(i+1, display1[i]);
    }
}

void update_display2(void)
{
	uint8_t i;

	for (i = 0; i < 8; i++)
	{
		max7219_writec2(i+1, display2[i]);
	}
}


void image1(const __flash uint8_t im[8])
{
    uint8_t i;

    for (i = 0; i < 8; i++)
	display1[i] = im[i];
}

void image2(const __flash uint8_t im[8])
{
	uint8_t i;

	for (i = 0; i < 8; i++)
	display2[i] = im[i];
}


void set_pixel1(uint8_t r, uint8_t c, uint8_t value)
{
    switch (value)
    {
    case 0: // Clear bit
	display1[r] &= (uint8_t) ~(0x80 >> c);
	break;
    case 1: // Set bit
	display1[r] |= (0x80 >> c);
	break;
    default: // XOR bit
	display1[r] ^= (0x80 >> c);
	break;
    }
}



void set_pixel2(uint8_t r, uint8_t c, uint8_t value)
{
	switch (value)
	{
		case 0: // Clear bit
		display2[r] &= (uint8_t) ~(0x80 >> c);
		break;
		case 1: // Set bit
		display2[r] |= (0x80 >> c);
		break;
		default: // XOR bit
		display2[r] ^= (0x80 >> c);
		break;
	}
}


void input_and_activate_pull_up(){
	DDRD &= ~(1 << PD0);
	DDRD &= ~(1 << PD1);
	DDRD &= ~(1 << PD2);
	
	DDRB &= ~(1 << PB0);
	
	
	DDRD &= ~(1 << PD4);
	DDRD &= ~(1 << PD5);
	DDRD &= ~(1 << PD6);
	DDRD &= ~(1 << PD7);
	
	PORTD |= (1 << PD0);
	PORTD |= (1 << PD1);
	PORTD |= (1 << PD2);
	
	PORTB |= (1 << PB0);


	PORTD |= (1 << PD5);
	PORTD |= (1 << PD6);
	PORTD |= (1 << PD7);

}



int main(void)
{
	
	DDRC != (1 << PC2);
	PORTC |= (1 << PC2);
	
	int finished_game = 0;
	
	int total_plane_parts_right = 0;
	int total_plane_parts_left = 0;
	
	int game_state = 0;
	
	int round_change = 0;
	
	LCD_init();
    uint8_t i, j;
	
	input_and_activate_pull_up();
	initialise_planes();

    TCNT1=0;
	OCR1A = 15;
	TCCR1A |= (1<<COM1A0);
	TCCR1B |= (1<<CS10) | (1<<WGM12);
	DDRB |= _BV(1);
	
	LCD_printAt(0x00, "PREGATITI AVIOANELE");
	
    max7219_init1();
	max7219_init2();
	


	
	int apasat_PD0 = 0;
	int apasat_PD1 = 0;
	
	int apasat_PD5 = 0;
	int apasat_PD6 = 0;
	
	int apasat_PB0 = 0;
	int apasat_PD7 = 0;
	
	while(1) {
		
		if ((PIND & (1 << PD0)) == 0) {
			if (!apasat_PD0) {
				if (left_cursor_x == 0) {
					set_pixel1(0, left_cursor_y, 0);
					left_cursor_x = 7;
				
				} else {
					set_pixel1(left_cursor_x, left_cursor_y,0);
					left_cursor_x--;
				}
				apasat_PD0 = 1;	
			}
		} else {
			
			apasat_PD0 = 0;
		}
		
		
		if ((PIND & (1 << PD1)) == 0) {
			if (!apasat_PD1) {	
				if (left_cursor_y == 7) {
					set_pixel1(left_cursor_x, 7, 0);
					left_cursor_y = 0;
						
				} else {
					set_pixel1(left_cursor_x, left_cursor_y,0);
					left_cursor_y++;
				}
				
				apasat_PD1 = 1;
			}		
		} else {
			
			apasat_PD1 = 0;
		}
		
		
		
		
		if ((PIND & (1 << PD5)) == 0) {
			if (!apasat_PD5) {
				if (right_cursor_x == 0) {
					set_pixel2(0, right_cursor_y, 0);
					right_cursor_x = 7;
					
				} else {
					set_pixel2(right_cursor_x, right_cursor_y,0);
					right_cursor_x--;
				}
			apasat_PD5 = 1;
			}
		} else {
	
			apasat_PD5 = 0;
		}
		
		
	
		if ((PIND & (1 << PD6)) == 0) {
			if (!apasat_PD6) {
				if (right_cursor_y == 7) {
					set_pixel2(right_cursor_x, 7, 0);
					right_cursor_y = 0;
					
				} else {
					set_pixel2(right_cursor_x, right_cursor_y,0);
					right_cursor_y++;
				}
				
				apasat_PD6 = 1;
			}
		} else {
			
			apasat_PD6 = 0;
		}
		

		if ((PINB & (1 << PB0)) == 0) {
			if (!apasat_PB0) {
				
				if (total_plane_parts_left != 8) {
					if (left_plane[left_cursor_x][left_cursor_y] == 0)
						total_plane_parts_left++;
					left_plane[left_cursor_x][left_cursor_y] = 1;
				}
				apasat_PB0 = 1;
					
				
			}
		} else {
		
			apasat_PB0 = 0 ;
		
		}
		
			
			
			
		if ((PIND & (1 << PD7)) == 0) {
			if (!apasat_PD7) {
				if (total_plane_parts_right != 8) {
					if (right_plane[right_cursor_x][right_cursor_y] == 0)
						total_plane_parts_right++;
				
					right_plane[right_cursor_x][right_cursor_y] = 1;
				}
					apasat_PD7 = 1;
			
			}
		} else {
			
			apasat_PD7 = 0;
		}
		
		
		
		if (show_left_cursor == 1) {
			set_pixel1(left_cursor_x,left_cursor_y, 2);
			
			
			
			
		}
			
		if (show_right_cursor == 1) {
			set_pixel2(right_cursor_x,right_cursor_y, 2);
		
		}
		
		
		if (total_plane_parts_right == 8 && total_plane_parts_left == 8)
			break;
		
		
		
		for (i = 0; i < 8; i++) {
			for (j = 0; j < 8 ; j++) {
				if(i!=left_cursor_x || j != left_cursor_y) {
					
					if (left_plane[i][j] == 1)
						set_pixel1(i, j, 1);
				}
				
				if(i!=right_cursor_x || j != right_cursor_y) {
									
					if (right_plane[i][j] == 1)
						set_pixel2(i, j, 1);
				}
				
			}	
		}
		
		
		update_display1();
		update_display2();
		_delay_ms(1000);
	}
	
	for (i = 0; i < 8; i++) {
		for (j = 0; j < 8 ; j++) {
			set_pixel1(i, j, 0);
			set_pixel2(i, j, 0);
					
		}
	}
	update_display1();
	update_display2();
	
	int left_right_turn = 0;
	
	int won_left = 0;
	int done_game = 0 ;
	int change_state;
	
	LCD_clear_top_line();
	LCD_printAt(0x00,"PREGATITI?");
	_delay_ms(50000);
	
		LCD_clear_top_line();
		LCD_printAt(0x00,"LEFT TURN");
	
		
	while(1) {
		change_state = 0;
		done_game = 1;
		if (left_right_turn == 0) {
			if ((PIND & (1 << PD5)) == 0) {
				if (!apasat_PD5) {
					if (right_cursor_x == 0) {
						set_pixel2(0, right_cursor_y, 0);
						right_cursor_x = 7;
								
					} else {
						set_pixel2(right_cursor_x, right_cursor_y,0);
						right_cursor_x--;
					}
					apasat_PD5 = 1;
					}
				} else {	
					apasat_PD5 = 0;
			}
					
					
					
			if ((PIND & (1 << PD6)) == 0) {
				if (!apasat_PD6) {
					if (right_cursor_y == 7) {
						set_pixel2(right_cursor_x, 7, 0);
						right_cursor_y = 0;
								
					} else {
						set_pixel2(right_cursor_x, right_cursor_y,0);
						right_cursor_y++;
					}
							
				apasat_PD6 = 1;
				}
			} else {		
				apasat_PD6 = 0;
			}
			
				if ((PIND & (1 << PD7)) == 0) {
					if (!apasat_PD7) {
						LCD_clear_top_line();
						LCD_printAt(0x00,"RIGHT TURN");
						if (left_plane[right_cursor_x][right_cursor_y] == 1) {
							left_plane[right_cursor_x][right_cursor_y] = 2;
							hit_matrix_right[right_cursor_x][right_cursor_y] = 2;	
						}
						else
							hit_matrix_right[right_cursor_x][right_cursor_y] = 1;
							
						change_state = 1;
						apasat_PD7 = 1;
								
					}
				} else {				
						apasat_PD7 = 0;
				}
			
			
			for (i = 0; i < 8; i++) {
				for (j = 0; j < 8; j++) {
					set_pixel1(i,j,left_plane[i][j]);
					if (right_cursor_x != i || right_cursor_y != j)
						set_pixel2(i,j,hit_matrix_right[i][j]);
				}
			}
				
				
			if (hit_matrix_right[right_cursor_x][right_cursor_y] == 0)
				set_pixel2(right_cursor_x,right_cursor_y, 2);
				
			update_display1();
			update_display2();
				
				
		} else {
			
			if ((PIND & (1 << PD0)) == 0) {
				if (!apasat_PD0) {
					if (left_cursor_x == 0) {
						set_pixel1(0, left_cursor_y, 0);
						left_cursor_x = 7;
			
					} else {
						set_pixel1(left_cursor_x, left_cursor_y,0);
						left_cursor_x--;
					}
					apasat_PD0 = 1;
				}
			} else {
				apasat_PD0 = 0;
			}



			if ((PIND & (1 << PD1)) == 0) {
				if (!apasat_PD1) {
					if (left_cursor_y == 7) {
						set_pixel1(left_cursor_x, 7, 0);
						left_cursor_y = 0;
			
					} else {
						set_pixel1(left_cursor_x, left_cursor_y,0);
						left_cursor_y++;
					}
					apasat_PD1 = 1;
				}
			} else {	
				apasat_PD1 = 0;
			}
			
			if ((PINB & (1 << PB0)) == 0) {
				if (!apasat_PB0) {
					LCD_clear_top_line();
					LCD_printAt(0x00,"LEFT TURN");
					if (right_plane[left_cursor_x][left_cursor_y] == 1) {
						right_plane[left_cursor_x][left_cursor_y] = 2;
						hit_matrix_left[left_cursor_x][left_cursor_y] = 2;

					}
				else
					hit_matrix_left[left_cursor_x][left_cursor_y] = 1;
				change_state = 1;
				apasat_PB0 = 1;
				}
			} else {
				apasat_PB0 = 0;
			}

		
		for (i = 0; i < 8; i++) {
			for (j = 0; j < 8; j++) {
				set_pixel2(i,j,right_plane[i][j]);
				if (left_cursor_x != i || left_cursor_y != j)
					set_pixel1(i,j,hit_matrix_left[i][j]);
			}
		}


		if (hit_matrix_left[left_cursor_x][left_cursor_y] == 0)
			set_pixel1(left_cursor_x,left_cursor_y, 2);

		update_display1();
		update_display2();			
	}
		
		
		for (i = 0; i < 8; i++) {
			for (j = 0; j < 8; j++) {
				if (left_plane[i][j] == 1)
					done_game = 0;
			}
		}	
		
		if (done_game == 1) {
			won_left = 0;
			break;
		}
		
		done_game = 1;
		for (i = 0; i < 8; i++) {
			for (j = 0; j < 8; j++) {
				if (right_plane[i][j] == 1)
					done_game = 0;
			}
		}
			
		if (done_game == 1) {
			won_left = 1;
			break;
		}		
		
		if (change_state == 1) {
			if (left_right_turn == 0)
				left_right_turn = 1;
			else
				left_right_turn = 0;
		}
			
		_delay_ms(1000);	
	}
	
	while (1) {
		if (won_left == 0) {
			image1(sad);
			image2(smile);	
			LCD_clear_top_line();
			LCD_printAt(0x00,"GRATZ LEFT");
		} else {
			image1(smile);
			image2(sad);
			LCD_clear_top_line();
			LCD_printAt(0x00,"GRATZ RIGHT");
		}
		update_display1();
		update_display2();
		_delay_ms(1000000);
		
	}
	
	
}
